@@ -1,5 +1,18 @@
+"""This is ServiceWall's package definition. See main.py for real content.
+"""
+import os
+import pickle
+#from pkgutil import extend_path
+#__path__ = extend_path(__path__, __name__)
+from servicewall.main import ServiceWall
+# update_service_defs needs to have service_helpers imported
+# to have working pickle in the ServiceWall class.
+from servicewall import service_helpers, _version
+
+
 # Making this directory a python package.
 name = "servicewall"
+version = _version.version
 __all__ = [
     "service_helpers",
     "network_helpers",
@@ -7,23 +20,14 @@ __all__ = [
     "statefulfirewall",
     "firewall",
 ]
-#from pkgutil import extend_path
-#__path__ = extend_path(__path__, __name__)
 
-# Load the main package
-from servicewall.main import ServiceWall
-
-# This function needs to be here and have service_helpers imported
-# this way to have working pickle in the ServiceWall class.
-from servicewall import service_helpers
 def update_service_defs():
-    import pickle
     service_pickle = "lib/services.p"
     service_defs_dir = "/etc/gufw/app_profiles"
-    s = service_helpers.scan_service_definitions(service_defs_dir)
+    service = service_helpers.scan_service_definitions(service_defs_dir)
     print("writing defs from %s to %s" % (service_defs_dir, service_pickle))
     if os.path.isfile(service_pickle):
         raise SystemError("There's already a file named %s" % service_pickle)
     with open(service_pickle, "wb") as fd:
-        pickle.dump(s, fd)
+        pickle.dump(service, fd)
 
